@@ -19,24 +19,37 @@ impl NcDirect {
     /// [notcurses_render()][c_api::notcurses_render]. This can be used to add
     /// color and styling to text in the standard output paradigm.
     ///
+    /// # Safety
+    /// You must not create multiple `NcDirect` instances at the same time, on
+    /// the same thread. You must [`stop`][NcDirect#method.stop] the current one
+    /// before creating a new one.
+    ///
     /// *C style function: [ncdirect_init()][c_api::ncdirect_init].*
-    pub fn new<'a>() -> NcResult<&'a mut NcDirect> {
+    pub unsafe fn new<'a>() -> NcResult<&'a mut NcDirect> {
         Self::with_flags(0)
     }
 
     /// New NcDirect with optional flags.
     ///
+    /// # Safety
+    /// You must not create multiple `NcDirect` instances at the same time, on
+    /// the same thread. You must [`stop`][NcDirect#method.stop] the current one
+    /// before creating a new one.
+    ///
     /// *C style function: [ncdirect_init()][c_api::ncdirect_init].*
-    pub fn with_flags<'a>(flags: NcDirectFlags) -> NcResult<&'a mut NcDirect> {
-        let res = unsafe { c_api::ncdirect_init(null(), null_mut(), flags) };
+    pub unsafe fn with_flags<'a>(flags: NcDirectFlags) -> NcResult<&'a mut NcDirect> {
+        let res = c_api::ncdirect_init(null(), null_mut(), flags);
         error_ref_mut![res, "Initializing NcDirect"]
     }
 
     /// Releases this NcDirect and any associated resources.
     ///
+    /// # Safety
+    /// You must not call this method repeatedly on the same `NcDirect` instance.
+    ///
     /// *C style function: [ncdirect_stop()][c_api::ncdirect_stop].*
-    pub fn stop(&mut self) -> NcResult<()> {
-        error![unsafe { c_api::ncdirect_stop(self) }, "NcDirect.stop()"]
+    pub unsafe fn stop(&mut self) -> NcResult<()> {
+        error![c_api::ncdirect_stop(self), "NcDirect.stop()"]
     }
 }
 
@@ -343,7 +356,7 @@ impl NcDirect {
     ///
     /// *C style function: [ncdirect_canopen_images()][c_api::ncdirect_canopen_images].*
     pub fn canopen_images(&self) -> bool {
-        unsafe { c_api::ncdirect_canopen_images(self) }
+        c_api::ncdirect_canopen_images(self)
     }
 
     /// Can we load videos?
@@ -561,12 +574,6 @@ impl NcDirect {
 
 /// ## NcDirect methods: I/O
 impl NcDirect {
-    #[doc(hidden)]
-    #[deprecated = "use `get` method instead"]
-    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
-        self.get(time, input)
-    }
-
     /// Returns a [char] representing a single unicode point.
     ///
     /// If an event is processed, the return value is the `id` field from that
@@ -602,11 +609,11 @@ impl NcDirect {
     ///
     /// In the case of a valid read, a [`char`] is returned.
     ///
-    /// *C style function: [ncdirect_getc_blocking()][c_api::ncdirect_getc_blocking].*
-    pub fn getc_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
-        let res = c_api::ncdirect_getc_blocking(self, input);
+    /// *C style function: [ncdirect_get_blocking()][c_api::ncdirect_get_blocking].*
+    pub fn get_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
+        let res = c_api::ncdirect_get_blocking(self, input);
         core::char::from_u32(res as u32)
-            .ok_or_else(|| NcError::with_msg(res, "NcDirect.getc_blocking()"))
+            .ok_or_else(|| NcError::with_msg(res, "NcDirect.get_blocking()"))
     }
 
     /// Reads input without blocking.
@@ -615,17 +622,17 @@ impl NcDirect {
     ///
     /// If no event is ready, returns 0.
     ///
-    /// *C style function: [ncdirect_getc_nblock()][c_api::ncdirect_getc_nblock].*
-    pub fn getc_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
-        let res = c_api::ncdirect_getc_nblock(self, input);
+    /// *C style function: [ncdirect_get_nblock()][c_api::ncdirect_get_nblock].*
+    pub fn get_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
+        let res = c_api::ncdirect_get_nblock(self, input);
         core::char::from_u32(res as u32)
-            .ok_or_else(|| NcError::with_msg(res, "NcDirect.getc_nblock()"))
+            .ok_or_else(|| NcError::with_msg(res, "NcDirect.get_nblock()"))
     }
 
     /// Get a file descriptor suitable for input event poll()ing.
     ///
     /// When this descriptor becomes available, you can call
-    /// [getc_nblock()][NcDirect#method.getc_nblock], and input ought be ready.
+    /// [get_nblock()][NcDirect#method.get_nblock], and input ought be ready.
     ///
     /// This file descriptor is not necessarily the file descriptor associated
     /// with stdin (but it might be!).
@@ -693,28 +700,18 @@ impl NcDirect {
         ll: NcChannels,
         lr: NcChannels,
         wchars: &[char; 6],
-        y_len: NcDim,
-        x_len: NcDim,
+        len_y: NcDim,
+        len_x: NcDim,
         ctlword: u32,
     ) -> NcResult<()> {
         error![
             unsafe {
                 let wchars = core::mem::transmute(wchars);
-                c_api::ncdirect_box(
-                    self,
-                    ul,
-                    ur,
-                    ll,
-                    lr,
-                    wchars,
-                    y_len as i32,
-                    x_len as i32,
-                    ctlword,
-                )
+                c_api::ncdirect_box(self, ul, ur, ll, lr, wchars, len_y, len_x, ctlword)
             },
             &format!(
                 "NcDirect.box({:0X}, {:0X}, {:0X}, {:0X}, {:?}, {}, {}, {})",
-                ul, ur, ll, lr, wchars, y_len, x_len, ctlword
+                ul, ur, ll, lr, wchars, len_y, len_x, ctlword
             )
         ]
     }
@@ -728,13 +725,11 @@ impl NcDirect {
         ur: NcChannels,
         ll: NcChannels,
         lr: NcChannels,
-        y_len: NcDim,
-        x_len: NcDim,
+        len_y: NcDim,
+        len_x: NcDim,
         ctlword: u32,
     ) -> NcResult<()> {
-        error![unsafe {
-            c_api::ncdirect_double_box(self, ul, ur, ll, lr, y_len as i32, x_len as i32, ctlword)
-        }]
+        error![unsafe { c_api::ncdirect_double_box(self, ul, ur, ll, lr, len_y, len_x, ctlword) }]
     }
 
     /// NcDirect.[box()][NcDirect#method.box] with the rounded box-drawing characters.
@@ -746,13 +741,11 @@ impl NcDirect {
         ur: NcChannels,
         ll: NcChannels,
         lr: NcChannels,
-        y_len: NcDim,
-        x_len: NcDim,
+        len_y: NcDim,
+        len_x: NcDim,
         ctlword: u32,
     ) -> NcResult<()> {
-        error![unsafe {
-            c_api::ncdirect_rounded_box(self, ul, ur, ll, lr, y_len as i32, x_len as i32, ctlword)
-        }]
+        error![unsafe { c_api::ncdirect_rounded_box(self, ul, ur, ll, lr, len_y, len_x, ctlword) }]
     }
 
     /// Draws horizontal lines using the specified [NcChannels]s, interpolating
