@@ -34,26 +34,6 @@ macro_rules! sleep {
     };
 }
 
-#[doc(hidden)]
-#[macro_export]
-#[deprecated]
-#[allow(unused_macros)]
-macro_rules! nrs {
-    ($($rest:tt)*) => {
-        nc_render_sleep!($($rest)*)
-    }
-}
-
-#[doc(hidden)]
-#[macro_export]
-#[deprecated]
-#[allow(unused_macros)]
-macro_rules! prs {
-    ($($rest:tt)*) => {
-        pile_render_sleep!($($rest)*)
-    }
-}
-
 /// [`Nc::render`][Nc#method.render]\(`$nc`\)? + [`sleep!`]`[$sleep_args]`.
 ///
 /// Renders the `$nc` [`Nc`]'s standard pile, and then,
@@ -102,7 +82,7 @@ macro_rules! pile_render_sleep {
 #[macro_export]
 macro_rules! visual_render_sleep {
     ($v: expr, $vo: expr, $nc:expr, $( $sleep_args:expr),+ ) => {
-        crate::NcVisual::blit($v, $nc, Some($vo))?;
+        unsafe { crate::NcVisual::blit($v, $nc, Some($vo))? };
         crate::Nc::render($nc)?;
         sleep![$( $sleep_args ),+];
     };
@@ -111,53 +91,6 @@ macro_rules! visual_render_sleep {
     };
 }
 
-/// [`NcDirect::flush`][NcDirect#method.flush]\(`$ncd`\)? + [`sleep!`]`[$sleep_args]`.
-///
-/// Flushes the `$ncd` [NcDirect] object and, if there's no error,
-/// calls the sleep macro with the rest of the arguments.
-///
-/// Returns [NcResult].
-#[macro_export]
-#[deprecated]
-#[doc(hidden)]
-macro_rules! fsleep {
-    ($ncd:expr, $( $sleep_args:expr),+ ) => {
-        // Rust style, with methods & NcResult
-        crate::NcDirect::flush($ncd)?;
-        sleep![$( $sleep_args ),+];
-    };
-    ($ncd:expr, $( $sleep_args:expr),+ ,) => {
-        rsleep![$ncd, $( $sleep_args ),* ]
-    };
-}
-
-#[deprecated]
-#[doc(hidden)]
-#[allow(unused_macros)]
-macro_rules! prsleep {
-    ($p:expr, $( $sleep_args:expr),+ ) => {
-        prs![$p, $( $sleep_args ),+];
-    };
-}
-
-#[deprecated]
-#[doc(hidden)]
-#[allow(unused_macros)]
-macro_rules! psleep {
-    ($p:expr, $( $sleep_args:expr),+ ) => {
-        prs![$p, $( $sleep_args ),+];
-    };
-}
-
-#[deprecated]
-#[doc(hidden)]
-#[allow(unused_macros)]
-macro_rules! rsleep {
-    ($nc:expr, $( $sleep_args:expr),+ ) => {
-        nrs![$nc, $( $sleep_args ),+];
-    };
-}
-
 // String & Print Macros -------------------------------------------------------
 
 /// Converts an `&str` into `*const c_char`.
@@ -167,6 +100,8 @@ macro_rules! rsleep {
 #[doc(hidden)]
 macro_rules! cstring {
     ($s:expr) => {
+        // NOTE: The returned pointer will be valid for as long as self is
+        // CHECK does this live long enough in all circumstances?
         std::ffi::CString::new($s).unwrap().as_ptr()
     };
 }
@@ -232,13 +167,13 @@ macro_rules! printf {
 /// ```
 /// # use libnotcurses_sys::*;
 /// # fn main() -> NcResult<()> {
-/// let nc = Nc::new_cli()?;
-/// let splane = nc.stdplane();
+/// let nc = unsafe { Nc::new_cli()? };
+/// let splane = unsafe { nc.stdplane() };
 /// splane.set_scrolling(true);
 /// putstr!(splane, "hello ")?;
 /// putstr!(splane, " world\n")?;
 /// putstr!(splane, "formatted text: {:?}\n", (0, 1.0, "two") )?;
-/// # nc.stop()?;
+/// # unsafe { nc.stop()? };
 /// # Ok(())
 /// # }
 /// ```
@@ -272,12 +207,12 @@ macro_rules! putstr {
 /// ```ignore
 /// # use libnotcurses_sys::*;
 /// # fn main() -> NcResult<()> {
-/// let nc = Nc::new_cli()?;
+/// let nc = unsafe { Nc::new_cli()? };
 /// let splane = nc.stdplane();
 /// splane.set_scrolling(true);
 /// putstrln!(splane, "hello world")?;
 /// putstrln!(splane, "formatted text: {:?}", (0, 1.0, "two") )?;
-/// # nc.stop()?;
+/// # unsafe { nc.stop()? };
 /// # Ok(())
 /// # }
 /// ```
